@@ -29,6 +29,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <signal.h>
+#include <stdlib.h>
 
 #include <gtk/gtk.h>
 #include <libgnomeprint/gnome-print-config.h>
@@ -40,6 +41,7 @@
 
 #define MAGIC_NUMBER -100
 
+gboolean option_list_tests;
 gint     num_test = MAGIC_NUMBER;
 gchar *  num_test_str = NULL;
 gboolean debug = FALSE;
@@ -54,6 +56,21 @@ typedef enum {
 	TEST_RETVAL_SUCCESS_LAST = 99,
 } TestRetval;
 
+poptContext popt;
+
+static struct poptOption options[] = {
+	{ "num", '\0', POPT_ARG_STRING, &num_test_str,   0,
+	  "Test number to run", "#"},
+	{ "list-tests", '\0', POPT_ARG_NONE, &option_list_tests,   0,
+	  "List on the console the description of the test cases", NULL},
+	{ "tree",     '\0', POPT_ARG_NONE, &tree,   0,
+	  "Show the tree of GPANodes",  NULL},
+	{ "debug",    '\0', POPT_ARG_NONE, &debug, 0,
+	  "Print debugging output",          NULL},
+	POPT_AUTOHELP
+	{ NULL }
+};
+
 typedef struct _TestCase TestCase;
 
 struct _TestCase {
@@ -63,31 +80,32 @@ struct _TestCase {
 
 #define max_num_test ((sizeof (test_cases) / sizeof (test_cases[0]))-1)
 
-static struct poptOption options[] = {
-	{ "num", '\0', POPT_ARG_STRING, &num_test_str,   0,
-	  "Test number to run", "#"},
-	{ "tree",     '\0', POPT_ARG_NONE, &tree,   0,
-	  "Show the tree of GPANodes",  NULL},
-	{ "debug",    '\0', POPT_ARG_NONE, &debug, 0,
-	  "Print debugging output",          NULL},
-	POPT_AUTOHELP
-	{ NULL }
-};
-
 static TestRetval test_simple (void);
 static TestRetval test_show (void);
 static TestRetval test_dialog_flags (void);
 static TestRetval test_paper_changes (void);
 static TestRetval test_printer_changes (void);
 static TestRetval test_multiple (void);
+static TestRetval test_print_ps (void);
+static TestRetval test_print_multiple_ps (void);
+static TestRetval test_print_pdf (void);
+static TestRetval test_print_multiple_pdf (void);
+static TestRetval test_print_multiple (void);
+static TestRetval test_paper_changes_print (void);
 
 static const TestCase test_cases[] = {
-	{ &test_simple,              "Simple checks."},
-	{ &test_show,                "show a print dialog"},
-	{ &test_dialog_flags,        "Shows a number of dialog with different flag convinationts"},
-	{ &test_paper_changes,       "Change paper parameters and monitor dialogs for changes"},
+	{ &test_simple,              "Simple checks"},
+	{ &test_show,                "Create and show a GnomePrintDialog"},
+	{ &test_dialog_flags,        "Shows dialogs with different flag options"},
+	{ &test_paper_changes,       "Change paper parameters and monitor dialog for changes"},
 	{ &test_printer_changes,     "Change the selected printer"},
-	{ &test_multiple,            "Create and destroy muliple GnomePrintDialogs."},
+	{ &test_multiple,            "Create and destroy multiple GnomePrintDialogs."},
+	{ &test_print_ps,            "Print to PS"},
+	{ &test_print_multiple_ps,   "Print multiple times to PS with the same GnomePrintConfig"},
+	{ &test_print_pdf,           "Print to PDF"},
+	{ &test_print_multiple_pdf,  "Print multiple times to PDF with the same GnomePrintConfig"},
+	{ &test_print_multiple,      "Print multiple times alternating between PS & PDF"},
+	{ &test_paper_changes_print, "Change paper parameters and print"},
 };
 
 /* Helper functions */
@@ -271,7 +289,7 @@ test_printer_changes (void)
 		g_warning ("The printers GtkOptionMenu didn't changed "
 			   "as expected, expected: %d changed: %d\n",
 			   len, callback_count);
-		if (FALSE) /* FIXME: Ignore for now, known issue */
+//		if (FALSE) /* FIXME: Ignore for now, known issue */
 		return TEST_RETVAL_ERROR;
 	}
 
@@ -338,7 +356,7 @@ test_paper_changes (void)
 	}
 
 	
-	test_run (1000);
+	test_run (100);
 
 	return TEST_RETVAL_SUCCESS;
 }
@@ -403,53 +421,58 @@ test_multiple (void)
 	GnomePrintJob *job;
 	GnomePrintConfig *config;
 	GtkWidget *dialog;
+	gint i, j;
+	gint max = 3;
+	gint run_for;
+
+	job = gnome_print_job_new (NULL);
+	config = gnome_print_job_get_config (job);
+	dialog = gnome_print_dialog_new (job, "Sample Print Dialog", 0);
+
+	if (!job || !config || !dialog)
+		return TEST_RETVAL_ERROR;
+
+	run_for = 1;
+
+	for (j = 0; j < 3; j++) {
+		
+		run_for = (j == 0 ? 1 : (j == 1 ? 100 : 400));
+		
+		for (i = 0; i < max; i++) {
+			gtk_widget_show (dialog);
+			test_run (run_for);
+			gtk_widget_destroy (dialog);
+			dialog = gnome_print_dialog_new (job, "Sample Print Dialog", 0);
+		}
+		
+		for (i = 0; i < max; i++) {
+			gtk_widget_show (dialog);
+			test_run (run_for);
+			gtk_widget_destroy (dialog);
+			g_object_unref (config);
+			g_object_unref (job);
+			config = gnome_print_config_default ();
+			job = gnome_print_job_new (config);
+			dialog = gnome_print_dialog_new (job, "Sample Print Dialog", 0);
+		}
+	}
+	
+	return TEST_RETVAL_SUCCESS;
+}
+
+static TestRetval
+test_paper_changes_print (void)
+{
+	GnomePrintConfig *config;
+	GnomePrintJob *job;
+	GtkWidget *dialog;
 	gint i;
 
-	config = gnome_print_config_default ();
-	if (!config)
-		return TEST_RETVAL_ERROR;
 	job = gnome_print_job_new (NULL);
-	if (!job)
+	config = gnome_print_job_get_config (job);
+
+	if (config == NULL)
 		return TEST_RETVAL_ERROR;
-	job = gnome_print_job_new (config);
-	if (!job)
-		return TEST_RETVAL_ERROR;
-	dialog = gnome_print_dialog_new (NULL, "Sample Print Dialog", 0);
-	if (!dialog)
-		return TEST_RETVAL_ERROR;
-
-	dialog = gnome_print_dialog_new (job, "Sample Print Dialog", 0);
-	if (!dialog)
-		return TEST_RETVAL_ERROR;
-	gtk_widget_show (dialog);
-
-	g_print ("a\n");
-	
-	for (i = 0; i < 1; i++) {
-		g_print ("b\n");
-//		gtk_widget_destroy (dialog);
-		/* This test needs love (Chema) */
-
-		g_print ("c\n");
-#if 1
-		config = gnome_print_config_default ();
-		job = gnome_print_job_new (config);
-#endif
-		g_print ("d\n");
-		
-		dialog = gnome_print_dialog_new (job, "Sample Print Dialog", 0);
-		if (!dialog)
-			return TEST_RETVAL_ERROR;
-
-		g_print ("e\n");
-		
-		gtk_widget_show (dialog);
-		test_run (1);
-		g_print ("f\n");
-	}
-	g_print ("g\n");
-
-	return TEST_RETVAL_SUCCESS;
 	
 	if (!gnome_print_config_set (config, GNOME_PRINT_KEY_PAPER_SIZE, "B3")) {
 		g_warning ("Could not change the Paper Size\n");
@@ -464,6 +487,7 @@ test_multiple (void)
 		return TEST_RETVAL_ERROR;
 	}
 
+	dialog = gnome_print_dialog_new (job, "Sample Print Dialog", 0);
 	for (i = 0; i < 15; i++) {
 		gtk_widget_destroy (dialog);
 		dialog = gnome_print_dialog_new (job, "Sample Print Dialog", 0);
@@ -501,6 +525,269 @@ test_simple (void)
 	return TEST_RETVAL_SUCCESS;
 }
 
+
+/* From gtkdialog.c. Nasty but is a gui test suite, what else do you expect? */
+typedef struct _ResponseData ResponseData;
+struct _ResponseData
+{
+  gint response_id;
+};
+
+static gboolean
+dialog_click (GtkWidget *dialog, gint action)
+{
+	GtkWidget *button = NULL;
+	GtkBox *box;
+	GList *list;
+	
+	box = GTK_BOX (GTK_DIALOG (dialog)->action_area);
+	list = box->children;
+	while (list) {
+		GtkBoxChild *child;
+		GtkWidget *widget;
+		child = list->data;
+		widget = child->widget;
+		if (GTK_IS_BUTTON (widget)) {
+			ResponseData *ad = g_object_get_data (G_OBJECT (widget),
+							      "gtk-dialog-response-data");
+			if (ad->response_id == action)
+				button = widget;
+		}
+		list = list->next;
+	}
+
+	if (!button) {
+		g_warning ("Could not find button to click\n");
+		return FALSE;
+	}
+
+	g_signal_emit_by_name (button, "clicked", 0);
+	
+	return FALSE;
+}
+
+static gboolean
+dialog_click_print_cb (gpointer dialog, gint action)
+{
+	return dialog_click (dialog, GNOME_PRINT_DIALOG_RESPONSE_PRINT);
+}
+		
+static TestRetval
+dialog_click_print (GtkWidget *dialog, gint msecs)
+{
+	tag = gtk_timeout_add (msecs, (GSourceFunc) dialog_click_print_cb, dialog);
+
+	return TEST_RETVAL_SUCCESS;
+}
+
+static void
+my_draw (GnomePrintContext *gpc)
+{
+	gnome_print_beginpage (gpc, "1");
+	gnome_print_moveto (gpc, 1, 1);
+	gnome_print_lineto (gpc, 200, 200);
+	gnome_print_stroke (gpc);
+	gnome_print_showpage (gpc);
+}
+
+
+static void
+my_print (GnomePrintJob *job, gboolean preview)
+{
+	GnomePrintContext *gpc;
+
+	gpc = gnome_print_job_get_context (job);
+	my_draw (gpc);
+	g_object_unref (G_OBJECT (gpc));
+	
+	gnome_print_job_close (job);
+
+	if (!preview) {
+		gnome_print_job_print (job);
+	}
+#if 0
+	else {
+		gtk_widget_show (gnome_print_job_preview_new (job, "Title goes here"));
+	}
+#endif
+}
+
+static TestRetval
+gui_print (GnomePrintJob *job, gint time)
+{
+	GtkWidget *dialog;
+	gint response;
+
+	dialog = gnome_print_dialog_new (NULL, "Sample Print Dialog", 0);
+	if (!dialog)
+		return TEST_RETVAL_ERROR;
+
+	gtk_widget_show (dialog);
+	test_run (2 * time);
+
+	if (dialog_click_print (dialog, 5 * time) != TEST_RETVAL_SUCCESS)
+		return TEST_RETVAL_ERROR;
+	
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	switch (response) {
+	case GNOME_PRINT_DIALOG_RESPONSE_PRINT:
+		my_print (job, FALSE);
+		break;
+	case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
+		my_print (job, TRUE);
+		break;
+	default:
+		return TEST_RETVAL_ERROR;
+	}
+
+	test_run (2 * time);
+	
+	return TEST_RETVAL_SUCCESS;
+}
+
+static TestRetval
+test_print_ps (void)
+{
+	GnomePrintConfig *config;
+	GnomePrintJob *job;
+	TestRetval retval;
+
+	config = gnome_print_config_default ();
+	job = gnome_print_job_new (config);
+	if (!job || !config)
+		return TEST_RETVAL_ERROR;
+
+	gnome_print_config_set (config, "Printer", "GENERIC");
+	retval = gui_print (job, 100);
+
+	return retval;
+}
+
+static TestRetval
+test_print_pdf (void)
+{
+	GnomePrintConfig *config;
+	GnomePrintJob *job;
+	TestRetval retval;
+
+	config = gnome_print_config_default ();
+	job = gnome_print_job_new (config);
+	if (!job || !config)
+		return TEST_RETVAL_ERROR;
+
+	gnome_print_config_set (config, "Printer", "PDF");
+
+	retval = gui_print (job, 300);
+	
+	return retval;
+}
+
+static TestRetval
+test_print_multiple_ps (void)
+{
+	GnomePrintConfig *config;
+	GnomePrintJob *job;
+	TestRetval retval;
+	gint i;
+	gint max = 4;
+
+	config = gnome_print_config_default ();
+	job = gnome_print_job_new (config);
+	if (!job || !config)
+		return TEST_RETVAL_ERROR;
+	
+	gnome_print_config_set (config, "Printer", "GENERIC");
+	
+	for (i = 0; i < max; i++) {
+		retval = gui_print (job, 100);
+		if (retval != TEST_RETVAL_SUCCESS)
+			return retval;
+	}
+		
+	return retval;
+}
+
+static TestRetval
+test_print_multiple_pdf (void)
+{
+	GnomePrintConfig *config;
+	GnomePrintJob *job;
+	TestRetval retval;
+	gint i;
+	gint max = 4;
+
+	/* First do it non-gui */
+	config = gnome_print_config_default ();
+	job = gnome_print_job_new (config);
+	if (!job || !config)
+		return TEST_RETVAL_ERROR;
+
+	gnome_print_config_set (config, "Printer", "PDF");
+	
+	for (i = 0; i < max; i++) {
+		my_print (job, FALSE);		
+	}
+	
+	for (i = 0; i < max; i++) {
+		retval = gui_print (job, 100);
+		if (retval != TEST_RETVAL_SUCCESS)
+			return retval;
+	}
+		
+	return retval;
+}
+
+static TestRetval
+test_print_multiple (void)
+{
+	GnomePrintConfig *config;
+	GnomePrintJob *job;
+	TestRetval retval;
+	gint i;
+	gint max = 2;
+
+	config = gnome_print_config_default ();
+	job = gnome_print_job_new (config);
+	if (!job || !config)
+		return TEST_RETVAL_ERROR;
+	
+	for (i = 0; i < max; i++) {
+		gnome_print_config_set (config, "Printer", "PDF");
+		retval = gui_print (job, 100);
+		if (retval != TEST_RETVAL_SUCCESS)
+			return retval;
+		
+		gnome_print_config_set (config, "Printer", "GENERIC");
+		retval = gui_print (job, 100);
+		if (retval != TEST_RETVAL_SUCCESS)
+			return retval;
+	}
+
+	g_object_unref (config);
+	g_object_unref (job);
+	
+	config = gnome_print_config_default ();
+	job = gnome_print_job_new (config);
+	
+	for (i = 0; i < max; i++) {
+		gnome_print_config_set (config, "Printer", "PDF");
+		retval = gui_print (job, 100);
+		if (retval != TEST_RETVAL_SUCCESS)
+			return retval;
+		
+		gnome_print_config_set (config, "Printer", "GENERIC");
+		retval = gui_print (job, 100);
+		if (retval != TEST_RETVAL_SUCCESS)
+			return retval;
+	}
+
+	g_object_unref (config);
+	g_object_unref (job);
+	
+	return retval;
+}
+
 static TestRetval
 test_dispatch (gint num_test)
 {
@@ -546,29 +833,43 @@ usage (gchar *error)
 {
 	g_print ("Error: %s\n\n", error);
 	g_print ("Usage: test-dialog --num=[num_test] [--tree]\n\n");
+	poptPrintHelp (popt, stdout, FALSE);
+	exit (TEST_RETVAL_BAD_PARAMETERS);
 	exit (TEST_RETVAL_BAD_PARAMETERS);
 }
 
 static void
 parse_command_line (int argc, const char ** argv, gint *num_test)
 {
-	poptContext popt;
-
+	gchar *env_test;
+	
 	popt = poptGetContext ("test_gpa", argc, argv, options, 0);
 	poptGetNextOpt (popt);
 
+	if (option_list_tests)
+		return;
+
+	
 	if (!num_test_str || num_test_str[0] == '\0') {
 		if (tree)
 			return;
-		usage ("num_test not specified");
-	}
+		env_test = getenv ("TEST_NUM");
+		if (env_test && env_test[0]) {
+			g_print ("TEST_NUM env variable = %s\n", env_test);
+			num_test_str = g_strdup (env_test);
+		} else {
+			usage ("num_test not specified");
+		}
+	} 
 
 	*num_test = atoi (num_test_str);
 
 	if (*num_test == -1) {
 		GList *list = NULL;
 		/* We crash, this is part of the sanity check that allows us verify that crashes
-		 * are treated as errors when they happen (Chema)*/
+		 * are treated as errors when they happen when running ./run-tests
+		 * (Chema)
+		 */
 		g_print ("Crashing ...\n");
 		list->next = NULL;
 	}
@@ -582,8 +883,16 @@ parse_command_line (int argc, const char ** argv, gint *num_test)
 	
 	if (debug)
 		g_print ("num_test is %d\n", *num_test);
-	
-	poptFreeContext (popt);
+}
+
+static void
+list_tests (void)
+{
+	gint i;
+	gint max = max_num_test + 1;
+
+	for (i = 0; i < max; i++)
+		g_print ("%s\n",  test_cases[i].description);
 }
 
 static void
@@ -605,10 +914,15 @@ main (int argc, const char * argv[])
 	parse_command_line (argc, argv, &num_test);
 
 	/* Catch sigsegv signals */
-	memset(&sig, 0, sizeof(sig));
+	memset (&sig, 0, sizeof(sigaction));
 	sig.sa_handler = handle_sigsegv;
 	sigaction (SIGSEGV, &sig, NULL);
 
+	if (option_list_tests) {
+		list_tests ();
+		return ret;
+	}
+	
 	if (tree)
 		test_dialog_tree ();
 
